// Copyright 2014 The Bazel Authors. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.devtools.build.lib.syntax;

/** Syntax node for a statement consisting of an expression evaluated for effect. */
public final class ExpressionStatement extends Statement {

  private final Expression expression;

  ExpressionStatement(FileLocations locs, Expression expression) {
    super(locs);
    this.expression = expression;
  }

  public Expression getExpression() {
    return expression;
  }

  @Override
  public void accept(NodeVisitor visitor) {
    visitor.visit(this);
  }

  @Override
  public int getStartOffset() {
    return expression.getStartOffset();
  }

  @Override
  public int getEndOffset() {
    return expression.getEndOffset();
  }

  @Override
  public Kind kind() {
    return Kind.EXPRESSION;
  }
}
